#coding: utf-8
'''

This software is licensed under the following agreement:

JHU Academic Software License Agreement

This license agreement ("License"), effective automatically by and between you (hereinafter referred to as the "Licensee") and The Johns Hopkins University (“JHU”) for use of the software known as CHASM (the “Software”) once and only so long as Licensee complies with the following terms and conditions:

   1. The requirement to acknowledge the copyright of JHU as follows: “Copyright Johns Hopkins University 2010, all rights reserved,” and copyrights of any incorporated third party software as described in the associated documentation.
   2. Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
         1. Redistributions of source code must retain the above copyright notice, and these terms and conditions.
         2. Neither the name of JHU nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission from an authorized JHU representative.
         3. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. LICENSEE AGREES TO DEFEND, INDEMNIFY AND HOLD HARMLESS JHU FOR ANY CLAIMS ARISING FROM LICENSEE’S USE OF THE SOFTWARE TO THE FULLEST EXTENT PERMITTED BY LAW.
   3. Only a non-exclusive, nontransferable license is granted to Licensee to use the Software for academic, non-profit, or government-sponsored research purposes. Use of the Software under this License is restricted to non-commercial purposes. Commercial use of the Software requires a separately executed written license agreement.
   4. Licensee agrees that it will use the Software, and any modifications, improvements, or derivatives to the Software that the Licensee may create (collectively, "Improvements") solely for non-commercial purposes and shall not distribute or transfer the Software or Improvements to any person or third parties without requiring that such third parties adhere to the terms of this License.
   5. Licensee acknowledges that JHU holds copyright in the Software or portions of the Software, and that the Software incorporates third party software which may be subject to additional terms and conditions. Licensee agrees that any Improvements made by Licensee shall be subject to the same terms and conditions as the Software.
   6. Licensee agrees that any publication of results obtained with the Software will acknowledge its use by an appropriate citation.
   7. Licensee’s rights under this License terminate automatically without notice from JHU if you fail to comply with any term(s) of this License.
   8. This License shall be governed by the laws of the State of Maryland, excluding the application of its conflicts of law rules. Licensee agrees that any dispute shall be appropriate only in the state and federal courts located within the State of Maryland.

Created May 17, 2010

Contact: chasm-users@lists.johnshopkins.edu
'''

import re, string
import sys
import os.path
from db.transcript_reader import *
from db.genomic_reader import *
from db.maf_reader import *

class DataSet:
    '''
    Dataset Object to keep track of files and classes
    Gets loaded when the XML is parsed

    MutationFileParser Function
    Initialize with location of the file
    Check that a line is valid using regular expression
    Return the parsed line: Transcript, Position, Wildtype, Mutation

    Verify if the mutation is valid
    MPO = MutationParserObject("some directory path")
    for Transcript, Position, Wildtype, Mutation in MPO.read():
        if dbhelper.validate(Transcript, Position, Wildtype, Mutation):

    If there is only 1 class, then by default no classes will be specified
    '''

    def __init__(self, indel=False):
        '''
        Constructor
        '''
        try:
            self.dataset = {} # {Class: [list of files]}
            self.indel = indel
        except Exception as e:
            raise

    def __iter__(self):
        try:
            return self
        except Exception as e:
            raise

    def next(self):
        while True:
            try:
                mutation = self.reader.next()
                if mutation != None:
                    if self.__isValidMutation(mutation):
                        return mutation
            except StopIteration:
                raise StopIteration
            except Exception as e:
                raise

    def setCoordinate(self, coordinate):
        try:
            self.coordinate = coordinate
        except Exception as e:
            raise

    def setFile(self, filename):
        try:
            # Counts the number of lines of the input mutation file.
            f = open(filename)
            self.no_lines = 0
            for line in f:
                if line.startswith('#'): continue
                self.no_lines += 1
            f.close()
            # there is a header for a MAF file
            if filename.endswith('.maf'): self.no_lines -= 1

            # Gets the 10% of total number of input lines.
            self.line_no_div = self.no_lines / 10

            if self.coordinate == 'transcript':
                self.reader = MutationReader(filename, self.indel)
                self.__isValidMutation = self.__isValidTranscript
            elif self.coordinate == 'genomic' and not filename.endswith('.maf'):
                self.reader = SNVReader(filename)
                self.__isValidMutation = self.__isValidGenomic
            elif self.coordinate == 'genomic':
                self.reader = MAFReader(filename)
                self.__isValidMutation = self.__isValidMaf
        except Exception, e:
            raise

    def addFile(self, datafile, dataclass=0):
        '''
        If the specified file exists
        If class exists, add datafile to existing class
        If class does not exist, creates a new class and adds datafile to that class
        Note: datafile here is the full path
        '''
        try:
            if os.path.isfile(datafile):
                if dataclass not in self.dataset.keys():
                    self.dataset[dataclass] = datafile
                else:
                    raise Exception("Two or more files assigned to same class. Please submit 1 file per class\n")
            else:
                raise IOError("Data file not found")
        except Exception as e:
            raise

    def getClasses(self):
        '''
        Function to output a sorted list of classes
        '''
        try:
            classes = self.dataset.keys()
            classes.sort()
            return classes
        except Exception as e:
            raise

    def getFiles(self, dataclass):
        '''
        Function to retrieve files for a particular class
        '''
        try:
            if dataclass in self.dataset.keys():
                return self.dataset.get(dataclass)
            else:
                return None
        except Exception as e:
            raise

    #def parseTranscriptFile(self, filename):
    #    '''
    #    Read missense mutations into a list
    #    '''
    #    #mutationdds = []
    #    for mutation in MutationReader(filename):
    #        if mutation != None:
    #            if self.__isValidTranscript(mutation, filename):
    #                mutations.append(mutation)
    #    return mutations

    def __isValidTranscript(self, mut):
        '''
        Check whether mutations are properly formatted
        '''
        try:
            status = False
            # Allow for inclusion of uniq ID but do not require it
            m1 = re.search("([A-Za-z0-9\\_\\-\\.]*)\t([A-Z])([0-9]*)([A-Z])",mut.line)
            m2 = re.search("([A-Za-z0-9\\_\\-\\.]*)\t([A-Za-z0-9\\_\\-\\.]*)\t([A-Z])([0-9]*)([A-Z])",mut.line)
            if m1 is not None or m2 is not None:
                status = True
            else:
                sys.stderr.write("Warning: Line " +str(mut.index)+ " is not formatted properly. The line will be skipped. \n")
            return status
        except Exception as e:
            raise

    #def parseGenomicFile(self, filename):
    #    '''
    #    Read genomic single nucleotide variants into a list
    #    '''
    #    snvs = []
    #    for snv in SNVReader(filename):
    #        if self.__isValidGenomic(snv, filename):
    #            snvs.append(snv)
    #    return snvs

    def __isValidGenomic(self, snv):
        '''
        Check whether genomic single nucleotide variants are properly formatted
        '''
        try:
            status = False
            # Allow for inclusion of uniq ID but do not require it
            m1 = re.search("([A-Za-z0-9]*)\t([0-9]*)\t([-|\+])\t(-|[A-Z]*)\t(-|[A-Z]*)$", snv.line)
            m2 = re.search("([A-Za-z0-9]*)\t([A-Za-z0-9]*)\t([0-9]*)\t([-|\+])\t(-|[A-Z]*)\t(-|[A-Z]*)$", snv.line)
            if m1 is not None or m2 is not None:
                status = True
            else:
                sys.stderr.write("Warning: Line " +str(snv.index)+ " is not formatted properly. The line will be skipped. \n")
            return status
        except Exception as e:
            raise

    def __isValidMaf(self, maf):
        return True

    def getMutations(self): # Currently only used for testing
        '''
        Generator function that parses all files and returns
        mutations and class label
        '''
        for dataclass in self.dataset.keys():
            currentdata = open(self.dataset.get(dataclass),"r")
            line = currentdata.readline()
            linenumber = 1
            while line != "":
                m = re.search("([A-Za-z0-9\\_\\-\\.]*)\t([A-Z])([0-9]*)([A-Z])",line)
                # If this mutation is contained within the database
                if m is not None:
                    yield m.group(1), int(m.group(3)), m.group(2), m.group(4), dataclass
                else:
                    sys.stderr.write("\t\"" + line + "\" is not a properly formatted mutation\n")
                line = currentdata.readline()
                linenumber += 1
            currentdata.close()


if __name__ == "__main__":
    '''
    Some test code
    '''
    ds = DataSet()
    ds.addFile("../example/Passengers.ids", "passenger")
    ds.addFile("../example/Drivers.ids", "driver")
    print ds.getFiles("passenger")
    print ds.getFiles("driver")
    for transcript, position, wildtype, mutation, dataclass in ds.getMutations():
        print dataclass
        print transcript
        print position
        print wildtype
        print mutation

