#coding: utf-8
'''

This software is licensed under the following agreement:

JHU Academic Software License Agreement

This license agreement ("License"), effective automatically by and between you (hereinafter referred to as the "Licensee") and The Johns Hopkins University (“JHU”) for use of the software known as CHASM (the “Software”) once and only so long as Licensee complies with the following terms and conditions:

   1. The requirement to acknowledge the copyright of JHU as follows: “Copyright Johns Hopkins University 2010, all rights reserved,” and copyrights of any incorporated third party software as described in the associated documentation.
   2. Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
         1. Redistributions of source code must retain the above copyright notice, and these terms and conditions.
         2. Neither the name of JHU nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission from an authorized JHU representative.
         3. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. LICENSEE AGREES TO DEFEND, INDEMNIFY AND HOLD HARMLESS JHU FOR ANY CLAIMS ARISING FROM LICENSEE’S USE OF THE SOFTWARE TO THE FULLEST EXTENT PERMITTED BY LAW.
   3. Only a non-exclusive, nontransferable license is granted to Licensee to use the Software for academic, non-profit, or government-sponsored research purposes. Use of the Software under this License is restricted to non-commercial purposes. Commercial use of the Software requires a separately executed written license agreement.
   4. Licensee agrees that it will use the Software, and any modifications, improvements, or derivatives to the Software that the Licensee may create (collectively, "Improvements") solely for non-commercial purposes and shall not distribute or transfer the Software or Improvements to any person or third parties without requiring that such third parties adhere to the terms of this License.
   5. Licensee acknowledges that JHU holds copyright in the Software or portions of the Software, and that the Software incorporates third party software which may be subject to additional terms and conditions. Licensee agrees that any Improvements made by Licensee shall be subject to the same terms and conditions as the Software.
   6. Licensee agrees that any publication of results obtained with the Software will acknowledge its use by an appropriate citation.
   7. Licensee’s rights under this License terminate automatically without notice from JHU if you fail to comply with any term(s) of this License.
   8. This License shall be governed by the laws of the State of Maryland, excluding the application of its conflicts of law rules. Licensee agrees that any dispute shall be appropriate only in the state and federal courts located within the State of Maryland.

Created August 17, 2011

Contact: chasm-users@lists.johnshopkins.edu
'''

#External lib imports
import sys
import random
import string
import time

#internal imports
from db.db_table import *
from db.db_row import *

# Vars
AminoAcids = ["A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y"]

class FeatureFetcher(object):
    """
    Object to retrieve amino acid substitution features from the SNVBOX database
    """
    def __init__(self, dbconn, filename, rawValues=False, allowMissing=False, questionMarkForMissingValues=False):
        self.rawValues = rawValues
        self.allowMissing = allowMissing
        self.questionMarkForMissingValues = questionMarkForMissingValues
        self.featureName = {}
        self.tableByFeature = {}
        self.featureMeans = {}
        self.featureRMSs = {}
        self.tableByName = {"AA_Features":AATable(),
                            "Regional_Comp":RCTable(),
                            "Local_Structure":LSTable(),
                            "Genomic_MSA":GMSATable(),
                            "Sam_MSA":SMSATable(),
                            "Exon_Features":ExonTable(),
                            "Uniprot_features":UniprotTable(),
                            "Triplet_Features":TripletTable(),
                            "Triplet_Sub_Features":TripletSubTable(),
                            "Indel_Features":IndelTable(),
                            "AlaSubPHC":AlaSubPhcTable()
                            } # For retrieval of table object by table name
        self.orderedList = ["AA_Features",
                            "Genomic_MSA",
                            "Sam_MSA",
                            "Exon_Features",
                            "Local_Structure",
                            "Regional_Comp",
                            "Triplet_Features",
                            "Triplet_Sub_Features",
                            "Uniprot_features",
                            "Indel_Features"
                            ]
        self.orderedFeatureList = []
        self.orderedFeatureNames = []
        self.__getSnvBoxFeatures(dbconn)
        self.__populateFromFeatureList(filename)
        self.__buildOrderedList()

    def __getSnvBoxFeatures(self, dbconn):
        """Store SNVbox feature mappings (SNVBox Table and column of table containing feature values)"""
        for row in FeatureTblRow.query(dbconn):
            self.featureName[row.get("Feature")] = row.get("Col_Name")
            self.tableByFeature[row.get("Feature")] = row.get("Table_Name")
            self.featureMeans[row.get("Feature")] = float(row.get("Mean"))
            self.featureRMSs[row.get("Feature")] = float(row.get("RMS"))

    def __populateFromFeatureList(self, filename):
        """
        Populate feature table objects with the features the user specified
        - Only lookup requested features
        """
        for feature in FeatureFileReader(filename):
            tableName = self.tableByFeature.get(feature)
            if tableName != None:
                table = self.tableByName.get(tableName)
                table.featurelist.append(self.featureName.get(feature))
                table.featureMean[feature] = self.featureMeans.get(feature)
                table.featureRMS[feature] = self.featureRMSs.get(feature)
                table.featurenames.append(feature)
            else:
                raise Exception(feature + ": No such feature in SNVBox\n")

    def __buildOrderedList(self):
        """Assign feature order - must be consistent for all arffs generated"""
        for tableName in self.orderedList:
            self.orderedFeatureNames += self.tableByName.get(tableName).featurenames
            self.orderedFeatureList += self.tableByName.get(tableName).featurelist

    def getFeaturesWithTranscriptCoordinate(self, dbconn, transcript, aa_pos, ref_aa, alt_aa):
        """
        Retrieve features for variant object
        -if rawValues=True return unscaled feature values
        -if allowMissing=True, return None for missing values
        """
        class EmptyClass:
            pass

        var = EmptyClass()
        var.acc = transcript
        var.aa_pos = aa_pos
        var.ref_aa = ref_aa
        var.alt_aa = alt_aa

        q = TranscriptFeatures.query(dbconn, var.acc)
        var.UID = q.get("UID")
        if var.UID != None:
            var.UID = str(var.UID)
            tsID = q.get("tsID")
            if self.__isTsVerMismatch(var.acc, tsID):
                sys.stderr.write("Warning: Refseq transcript version number (" + transcript + ") does not match Refseq version in database (" + tsID + ")\n")
            feats = []
            featNames = []
            for tableName in self.orderedList:
                if len(self.tableByName.get(tableName).featurelist) > 0:
                    feats += self.tableByName.get(tableName).getFeatures(dbconn, var)
                    featNames += self.tableByName.get(tableName).featurenames
            if not self.rawValues:
                feats = self.__scaleValues(feats, featNames)
            if not self.allowMissing and None in feats:
                feats = self.__fillMissingValues(feats)
            featDic = {}
            for i in xrange(len(feats)):
                featDic[self.orderedFeatureNames[i]] = feats[i]
            transcriptDic = {}
            transcriptDic['aaLen'] = q.get("aaLen")
            transcriptDic['UID'] = q.get('UID')
            transcriptDic['tsID'] = q.get('tsID')
        else:
            sys.stderr.write(var.uid + " not found in SNVbox database\n")
            exit()
            feats = None
            featDic = None
            transcriptDic = None

        return featDic, transcriptDic

    def getFeatures(self, dbconn, var):
        """
        Retrieve features for variant object
        -if rawValues=True return unscaled feature values
        -if allowMissing=True, return None for missing values
        """
        if hasattr(var, 'sequence_ontology') and var.sequence_ontology in ['MS', 'SY']:
            if not var.alt_aa in AminoAcids or not var.ref_aa in AminoAcids:
                return None

        q = TranscriptFeatures.query(dbconn, var.acc)
        var.UID = q.get("UID")
        if var.UID != None:
            var.UID = str(var.UID)
            tsID = q.get("tsID")

            var.aa_len = str(q.get("aaLen"))

            if self.__isTsVerMismatch(var.acc, tsID):
                sys.stderr.write("Warning: Refseq transcript version number (" + var.acc + ") does not match Refseq version in database (" + tsID + ")\n")
            feats = []
            featNames = []
            for tableName in self.orderedList:

                if len(self.tableByName.get(tableName).featurelist) > 0:
                    feats += self.tableByName.get(tableName).getFeatures(dbconn, var)
                    featNames += self.tableByName.get(tableName).featurenames
            if not self.rawValues:
                feats = self.__scaleValues(feats, featNames)
            if self.allowMissing and self.questionMarkForMissingValues:
                feats = self.__changeNoneValuesToQuestionMarks(feats)
            if not self.allowMissing and None in feats:
                feats = self.__fillMissingValues(feats)
        else:
            if var.genomicstatus == 1:
                sys.stderr.write(var.uid + " " + \
                                    var.chrom + ":" + var.start + "-" + var.end + " " + \
                                    var.strand + " "  + \
                                    var.ref_aa + ">" + var.alt_aa + \
                                    " mapped to " + var.acc + " " + \
                                    var.aaSubst + " was not found in SNVbox database\n")
            else:
                sys.stderr.write(var.uid + " " + \
                                    var.acc + " " + \
                                    var.ref_aa + var.aa_pos + var.alt_aa + \
                                    " not found in SNVbox database\n")
            feats = None
        return feats

    def __isTsVerMismatch(self, varID, dbID):
        """Check for refseq version mismatch """
        mismatch = False
        if varID.find(".") != -1 and not varID.startswith("CCDS"):
            ver = varID.split(".")[-1]
            dbver = dbID.split(".")[-1]
            if ver != dbver:
                mismatch = True
        return mismatch

    def __scaleValues(self, feats, featNames):
        """Scale features by subtracting mean and dividing by RMS.
           -999.0 Mean means no scaling.
        """
        for i in xrange(len(feats)):
            featMean = self.featureMeans.get(featNames[i])
            featRMS = self.featureRMSs.get(featNames[i])
            if feats[i] != None:
                if featRMS == 0:
                    feats[i] = 0.0
                else:
                    feats[i] = str((float(feats[i])-featMean)/featRMS)
        return feats

    def __fillMissingValues(self, feats):
        """Fill missing values - can fill with mean raw value or mean after scaling ( =0 )"""
        for i in xrange(len(feats)):
            featureName = self.orderedFeatureList[i]
            if feats[i] == None:
                if self.rawValues:
                    feats[i] = str(self.featureMeans.get(self.orderedFeatureList[i]))
                else:
                    feats[i] = "0.0" # Assumes mean value post-scaling is 0
        return feats

    def __changeNoneValuesToQuestionMarks(self, feats):
        """Fill missing values - can fill with mean raw value or mean after scaling ( =0 )"""
        for i in xrange(len(feats)):
            if feats[i] == None:
                if self.questionMarkForMissingValues:
                    feats[i] = '?'
        return feats


class FeatureFileReader(object):
    '''
    Object to read featurelist file
    '''
    def __init__(self, filename):
        self.fh = file(filename, 'r')

    def __iter__(self):
        return self

    def next(self):
        while True:
            line = self.fh.readline()
            if line == "":
                self.fh.close()
                raise StopIteration
            return line.strip()


class FeatureTblRow(DbRowBase):

    @staticmethod
    def query(conn):
        cursor=conn.cursor()
        cursor.execute("Select * from Features;")
        while True:
            row = cursor.fetchone()
            if row == None:
                break
            yield(FeatureTblRow(row, cursor.description))


if __name__ == "__main__":
    pass
