import db
import csv
from db.transcript_reader import *
from db.DBUtil import DBUtil
from db.CodonMap import *
import argparse


def parse_arguments():
    info = 'Get a DNA mutation that leads to the provided protein change'
    parser = argparse.ArgumentParser(description=info)
    parser.add_argument('-i', '--input',
                        type=str, required=True,
                        help='SNVBox transcript input format')
    parser.add_argument('-o', '--output',
                        type=str, required=True,
                        help='Output SNVBox genomic format')
    args = parser.parse_args()
    return vars(args)


def codon_row_to_genomic(codon_row, mut):
    codon = Codon(codon_row)
    codon_nucs = codon.nucs
    possible_nucs = set(['A', 'T', 'C', 'G'])
    output_list = []
    ###############################
    # try single nucleotide changes
    ###############################
    for i in range(3):
        possible_mut_nucs = possible_nucs - set([codon_nucs[i]])
        for n in possible_mut_nucs:
            # make new codon with mutation
            letters = list(codon_nucs)
            letters[i] = n
            new_codon = ''.join(letters)

            # check if it equals alt aa
            if mut.alt_aa == codonTable[new_codon]:
                # get info on mutation
                ref_nuc = codon.nucs[i]
                alt_nuc = n
                chrom = codon.chrom
                uid = mut.uid
                if i == 0:
                    pos = codon.pos1
                elif i == 1:
                    pos = codon.pos2
                elif i == 2:
                    pos = codon.pos3
                return [uid, chrom, pos, codon.strand, ref_nuc, alt_nuc]
    ##################################
    # try dinucleotide changes if SNV doesn't work
    ##################################
    for i in range(2):
        possible_mut_nucs1 = possible_nucs - set([codon_nucs[i]])
        possible_mut_nucs2 = possible_nucs - set([codon_nucs[i+1]])
        for n1 in possible_mut_nucs1:
            for n2 in possible_mut_nucs2:
                # make new codon with mutation
                letters = list(codon_nucs)
                letters[i] = n1
                letters[i+1] = n2
                new_codon = ''.join(letters)

                # check if it equals alt aa
                if mut.alt_aa == codonTable[new_codon]:
                    # get info on mutation
                    if codon.strand == '+':
                        ref_nuc = codon.nucs[i] + codon.nucs[i+1]
                        alt_nuc = n1 + n2
                    else:
                        ref_nuc =  codon.nucs[i+1] + codon.nucs[i]
                        alt_nuc = n2 + n1
                    chrom = codon.chrom
                    uid = mut.uid
                    # figure out the position
                    if i == 0:
                        pos = min(codon.pos1, codon.pos2)
                    elif i == 1:
                        pos = min(codon.pos3, codon.pos2)
                    return [uid, chrom, pos, codon.strand, ref_nuc, alt_nuc]


def main(opts):
    # setup db connection and mutation reader
    dbu = DBUtil()
    mutations = MutationReader(opts['input'], False)

    # iterate over each mutation
    output_list = []
    for mut in mutations:
        # get codon object
        codon_rows = CodonTblRow.query_mutation(dbu.conn, mut)
        codon_row = next(codon_rows)

        # figure out if there is reference aa problem
        ref_aa = codonTable[codon_row.bases]
        if ref_aa != mut.ref_aa:
            print('Reference AA in mutation ({0}) is not the same as in transcript {1} ({2})'.format(mut.ref_aa, mut.acc,  ref_aa))
            continue

        # figure out genomic coordinates
        genomic_line = codon_row_to_genomic(codon_row, mut)
        genomic_line[2] = str(genomic_line[2])  # convert pos to string

        # add line to output
        output_list.append(genomic_line)

    # save results
    with open(opts['output'], 'w') as handle:
        mywriter = csv.writer(handle, delimiter='\t', lineterminator='\n')
        mywriter.writerows(output_list)


if __name__ == '__main__':
    opts = parse_arguments()
    main(opts)
